package com.sun.org.apache.bcel.internal.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public class BootstrapMethods extends Attribute
{
    private BootstrapMethod[] bootstrap_methods;
    
    public BootstrapMethods(BootstrapMethods c) {
        this(c.getNameIndex(), c.getLength(), c.getBootstrapMethods(), c.getConstantPool());
    }
    
    public BootstrapMethods(int name_index, int length, BootstrapMethod[] bootstrap_methods, ConstantPool constant_pool) {
        super((byte)20, name_index, length, constant_pool);
        this.bootstrap_methods = bootstrap_methods;
    }
    
    BootstrapMethods(int name_index, int length, DataInput input, ConstantPool constant_pool) throws IOException {
        this(name_index, length, (BootstrapMethod[])null, constant_pool);
        int num_bootstrap_methods = input.readUnsignedShort();
        this.bootstrap_methods = new BootstrapMethod[num_bootstrap_methods];
        for (int i = 0; i < num_bootstrap_methods; ++i) {
            this.bootstrap_methods[i] = new BootstrapMethod(input);
        }
    }
    
    public final BootstrapMethod[] getBootstrapMethods() {
        return this.bootstrap_methods;
    }
    
    public final void setBootstrapMethods(BootstrapMethod[] bootstrap_methods) {
        this.bootstrap_methods = bootstrap_methods;
    }
    
    @Override
    public void accept(Visitor v) {
        v.visitBootstrapMethods(this);
    }
    
    @Override
    public BootstrapMethods copy(ConstantPool _constant_pool) {
        BootstrapMethods c = (BootstrapMethods)this.clone();
        c.bootstrap_methods = new BootstrapMethod[this.bootstrap_methods.length];
        for (int i = 0; i < this.bootstrap_methods.length; ++i) {
            c.bootstrap_methods[i] = this.bootstrap_methods[i].copy();
        }
        c.setConstantPool(_constant_pool);
        return c;
    }
    
    @Override
    public final void dump(DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.bootstrap_methods.length);
        for (BootstrapMethod bootstrap_method : this.bootstrap_methods) {
            bootstrap_method.dump(file);
        }
    }
    
    @Override
    public final String toString() {
        StringBuilder buf = new StringBuilder();
        buf.append("BootstrapMethods(");
        buf.append(this.bootstrap_methods.length);
        buf.append("):\n");
        for (int i = 0; i < this.bootstrap_methods.length; ++i) {
            buf.append("  ").append(i).append(": ");
            buf.append(this.bootstrap_methods[i].toString(super.getConstantPool())).append("\n");
        }
        return buf.toString();
    }
}
